﻿package {
	import flash.desktop.*;
	import flash.display.*;
	import flash.events.*;
	import flash.filesystem.*;
	import flash.filters.*;
	import flash.geom.*;
	import flash.net.*;
	import flash.system.*;
	import flash.utils.*;
	import com.adobe.images.JPGEncoder;
	
	public class MonochromeGenerator extends Sprite {
		
		private var _thumb:Sprite;
		private var _file:File;
		private var _tmpFile:File;
		private var _tmpDir:File;
		private var _srcImg:BitmapData;
		private var _ldr:Loader;
		
		//コンストラクタ
		public function MonochromeGenerator() {
			stage.nativeWindow.addEventListener(Event.CLOSE, closeHandler);
			init();
		}
		
		private function init() {
			//AIRアプリ内にドラッグインしたときに
			_canvas.addEventListener(NativeDragEvent.NATIVE_DRAG_ENTER, dragEnterHandler);
			//AIRアプリ内にドロップしたときに
			_canvas.addEventListener(NativeDragEvent.NATIVE_DRAG_DROP, dragDropHandler);
		}
		
		//ドラッグイン
		private function dragEnterHandler(evt:NativeDragEvent):void {
			if (evt.clipboard.hasFormat(ClipboardFormats.FILE_LIST_FORMAT)) {
				var file:File = evt.clipboard.getData(ClipboardFormats.FILE_LIST_FORMAT)[0];
				if (checkKind(file.url)) {
					//キャンバスへのドラッグインを許可
					NativeDragManager.acceptDragDrop(_canvas);
				}
			}
		}
		
		//拡張子チェック
		private function checkKind(filePath:String):Boolean {
			var kinds:Array = [".jpg", ".jpeg"];
			for (var i:int = 0; i < kinds.length; i++) {
				var chkKind:String = kinds[i];
				var pattern:RegExp = new RegExp(chkKind, "i");
				var chkStr:String = filePath.substr(filePath.length - chkKind.length, filePath.length);
				if (pattern.exec(chkStr) != null) return true;
			}
			return false;
		}

		//AIRアプリ内にドロップ、画像読み込み処理
		private function dragDropHandler(evt:NativeDragEvent):void {
			// 連続してドラッグされないようにリスナー削除
			_canvas.removeEventListener(NativeDragEvent.NATIVE_DRAG_ENTER, dragEnterHandler);
			_canvas.removeEventListener(NativeDragEvent.NATIVE_DRAG_DROP, dragDropHandler);
			
			_file = evt.clipboard.getData(ClipboardFormats.FILE_LIST_FORMAT)[0];					
			var stream:FileStream = new FileStream();
			stream.open(_file, FileMode.READ);
			try {
				var byteArr:ByteArray = new ByteArray();
				stream.readBytes(byteArr, 0, stream.bytesAvailable);
				_ldr = new Loader();
				_ldr.contentLoaderInfo.addEventListener(Event.COMPLETE, loadImgCompHandler);
				_ldr.loadBytes(byteArr);
			} catch (error:Error) {
				stage.nativeWindow.close();
			} finally {
				stream.close();
			}
		}
		
		//画像読み込み完了
		private function loadImgCompHandler(evt:Event):void {
			// 画像を白黒にする
			var myColorMatrix:ColorMatrixFilter = new ColorMatrixFilter();
			
			myColorMatrix.matrix = [0.3,0.59,0.11,0,0,
											   0.3,0.59,0.11,0,0,
											   0.3,0.59,0.11,0,0,
											   0,0,0,1,0];
			
			_ldr.filters = [myColorMatrix];
			
			//サムネイルを作成
			var w = 200/ _ldr.width;
			var h = 200/ _ldr.height;
			var ratio = (w < h)? w:h;
			_thumb = new Sprite();
			_thumb.scaleX = _thumb.scaleY = ratio;
			
			//キャンバス上に画像を配置
			_thumb.addChild(_ldr);
			_canvas.addChild(_thumb);
			
			// 白黒にした画像を一時保存ディレクトリに書き出し
			_srcImg = new BitmapData(_ldr.width, _ldr.height);
			_srcImg.draw(_ldr);
			var jpgEnc:JPGEncoder = new JPGEncoder(80);
			var jpgBytes:ByteArray = jpgEnc.encode(_srcImg);
			_tmpDir = File.createTempDirectory();
			_tmpFile = _tmpDir.resolvePath(_file.name.substr(0, -4) + "_gray.jpg");
			var stream:FileStream = new FileStream();
			stream.open(_tmpFile, FileMode.WRITE);
			stream.writeBytes(jpgBytes, 0, jpgBytes.length);
			stream.close();
			
			_thumb.addEventListener(MouseEvent.MOUSE_DOWN, mouseDownHandler);
		}
		
		//ドラッグ開始
		private function mouseDownHandler(evt:MouseEvent):void {
			//クリップボードに書き出すファイルを追加
			var cb:Clipboard=new Clipboard();
			cb.setData(ClipboardFormats.FILE_LIST_FORMAT, [_tmpFile]);
			
			// Drag 時のイメージ
			var dragImg:BitmapData = new BitmapData(_thumb.width / 2, _thumb.height / 2);
			dragImg.draw(_thumb, new Matrix(_thumb.scaleX / 2,0,0,_thumb.scaleY / 2,0,0));
			
			//マウスポインタとBitmapDataの重なる位置を調整
			var offset:Point = new Point(-dragImg.width/2, -dragImg.height/2);
			
			//ドラッグの開始処理
			NativeDragManager.doDrag(_canvas, cb, dragImg, offset);
			
			//ドラッグ&ドロップ処理の完了、中止時に対するリスナーを登録
			_canvas.addEventListener(NativeDragEvent.NATIVE_DRAG_COMPLETE ,
									 													dragCompHandler);
		}
		
		//ドラッグ&ドロップ処理の完了、中止時
		private function dragCompHandler(evt:Event) {
			_canvas.removeEventListener(NativeDragEvent.NATIVE_DRAG_COMPLETE ,
																							dragCompHandler);
			_canvas.removeChild(_thumb);			
			_tmpDir.deleteDirectory(true);//一時的ファイルを削除
			init();
		}
		
		//AIRアプリケーションを閉じたとき
		private function closeHandler(evt:Event) {
			//一時的ファイルが存在する場合は削除
			if (_tmpDir && _tmpDir.exists) _tmpDir.deleteDirectory(true);
		}
	}
}


