﻿package {
	
	import flash.display.*;
	import flash.data.*;
	import flash.events.*;
	import flash.filesystem.File;
	import flash.text.*;
	
	/**
	  *  ToDo リスト
	  */
	public class ToDoList extends MovieClip {
		
		private var _taskList:Array;
		private var _connection:SQLConnection;
		private var _selectStmt:SQLStatement;
		
		// コンストラクタ
		public function ToDoList() {
			this.stage.scaleMode = StageScaleMode.NO_SCALE;
			this.stage.align = StageAlign.TOP_LEFT;
			_closeBtn.addEventListener(MouseEvent.CLICK, function(evt:Event) {stage.nativeWindow.close();});
			_inputBtn.addEventListener(MouseEvent.CLICK, insertTask);
			initDB();
			_bg.addEventListener(MouseEvent.MOUSE_DOWN, windowMove);
		}
		
		// DB の初期化
		private function initDB() {
			trace("dir : " + File.applicationStorageDirectory.nativePath);
			var file:File = File.applicationStorageDirectory.resolvePath("ToDoList.db");
			_connection = new SQLConnection();
			_connection.addEventListener(SQLEvent.OPEN, connectionOpenHandler);
			_connection.addEventListener(SQLErrorEvent.ERROR, dbErrorHandler);
			_connection.open(file);
		}
		
		// DB コネクションオープン
		public function connectionOpenHandler(evt:Event):void {
			createTable();
		}
		
		// DB エラーハンドラー
		public function dbErrorHandler(evt:Event):void {
			trace("DB Err :: " + evt);
		}
		
		// テーブルの作成
		private function createTable():void {
			var stmt = new SQLStatement();
			stmt.sqlConnection = _connection;
			stmt.text=
				"CREATE TABLE IF NOT EXISTS task ("+
				" no INTEGER PRIMARY KEY,"+
				" status BOOLEAN,"+
				" desc TEXT" +
			")";
			stmt.addEventListener(SQLEvent.RESULT, stmtCreateResult);
			stmt.addEventListener(SQLErrorEvent.ERROR, dbErrorHandler);
			stmt.execute();
		}
		
		// SQL 実行完了
		private function stmtCreateResult(evt:Event):void {
			searchTask();
		}
		
		// タスクを検索
		private function searchTask() {
			_selectStmt = new SQLStatement();
			_selectStmt.sqlConnection = _connection;
			_selectStmt.text = "SELECT no, status, desc FROM task";
			_selectStmt.addEventListener(SQLEvent.RESULT, setTaskList);
			_selectStmt.addEventListener(SQLErrorEvent.ERROR, dbErrorHandler);
			_selectStmt.execute();
		}
		
		// タスクを表示
		private function setTaskList(evt:Event) {
			if (_taskList) {
				for (var i:Number = 0; i < _taskList.length; i++) {
					var delTask:Task = Task(_taskList[i]);
					this.removeChild(delTask);
				}
			}
			var res:SQLResult = _selectStmt.getResult();
			if (res.data != null) {
				_taskList = new Array();
				for (var j:Number = 0; j < res.data.length; j++) {
					var task:Task = new Task(res.data[j], deleteTaskHandler, changeStatusHandler);
					this.addChild(task);
					task.y = j * (task.height + 5) + 85;
					_taskList.push(task);
				}
				this.stage.nativeWindow.height = res.data.length * (task.height + 5) + 80;
			} else {
				_taskList = null;
				this.stage.nativeWindow.height = 80;
			}
		}
	
		//タスクの追加
		private function insertTask(evt:Event):void {
			var stmt = new SQLStatement();
			stmt.sqlConnection = _connection;
			stmt.text = "INSERT INTO task (no, status, desc)"
							+ "VALUES (Null, 'false', '"+ _txtInput.text +"')";
			stmt.addEventListener(SQLEvent.RESULT, insertTaskResult);
			stmt.addEventListener(SQLErrorEvent.ERROR, dbErrorHandler);
			stmt.execute();
		}
		
		//タスクの追加完了
		private function insertTaskResult(evt:Event) {
			_txtInput.text = "";
			searchTask();
		}
		
		// ステータス変更
		public function changeStatusHandler(id:Number, changeStatus:Number) {
			var stmt = new SQLStatement();
			stmt.sqlConnection = _connection;
			stmt.text="UPDATE task SET status=" + changeStatus + " WHERE no=" + id;
			stmt.addEventListener(SQLEvent.RESULT, stmtCreateResult);
			stmt.addEventListener(SQLErrorEvent.ERROR, dbErrorHandler);
			stmt.execute();
		}
		
		// タスク削除
		public function deleteTaskHandler(id:Number) {
			var stmt = new SQLStatement();
			stmt.sqlConnection = _connection;
			stmt.text="DELETE FROM task WHERE no=" + id;
			stmt.addEventListener(SQLEvent.RESULT, stmtCreateResult);
			stmt.addEventListener(SQLErrorEvent.ERROR, dbErrorHandler);
			stmt.execute();
		}
		
		// パーツのドラッグ処理
		private function windowMove(event:MouseEvent):void {
			stage.nativeWindow.startMove();
		}
	}
}