﻿package {
	
	import caurina.transitions.*;
	import caurina.transitions.properties.*;
	import com.adobe.utils.*;
	import flash.display.*;
	import flash.events.*;
	import flash.filesystem.*;
	import flash.media.*;
	import flash.text.*;
	import flash.utils.*;
	
	/**
	 * エンドロールを表示するクラス
	 */
	public class RollPanel extends Sprite {
		
		// 表示するノード
		private var _imgLoadCnt:Number = 0;
		private var _imgNodeList:Array;
		private var _musicLoadCnt:Number = 0;
		private var _musicNodeList:Array;
		private var _otherList:Array;
		
		// ノードを載せて移動させるパネル
		private var _panel:Sprite;
		
		// 表示する画像
		private var _currentImg:ImgNode;
		private var _currentImgNo:Number = 0;
		
		// 再生する音声ファイル
		private var _currentMusic:SoundChannel;
		private var _currentMusicNo:Number = 0;
		
		// 黒い背景
		private var _rollBG:RollBG;
		
		// ロールパネルを閉じるボタン
		private var _closeBtn:CloseBtn;
		
		// ウィンドウ
		private var _window:NativeWindow;
		
		// コンストラクタ
		public function RollPanel() {
			
			this.alpha = 0;
			
			// エンドロールを表示するウィンドウを開く
			var options:NativeWindowInitOptions = new NativeWindowInitOptions();
			options.systemChrome = NativeWindowSystemChrome.NONE;
			options.transparent = true;
			
			_window = new NativeWindow(options);
			_window.alwaysInFront = true;
			_window.maximize();
			_window.title = "EndRoll";
			_window.activate();
			
			// ステージを最大サイズで開けるように初期化
			_window.stage.scaleMode = StageScaleMode.NO_SCALE;
			_window.stage.align = StageAlign.TOP_LEFT;
			_window.stage.addChild(this);
			
			// クローズ処理の実装
			_window.addEventListener(Event.CLOSE, closeWindowHandler, false, 0, true);
		}
		
		// ウィンドウを閉じる際、イベントを中継する
		private function closeWindowHandler(e:Event ) {
			dispatchEvent(e);
		}
		
		// ボタンによる閉じる処理
		private function closeBtnHandler(e:Event) {
			_window.close();
		}
		
		// 表示の初期化を行う
		public function init(list:Array) {
			
			// 背景の生成
			_rollBG = new RollBG;
			this.addChild(_rollBG);
			_rollBG._bg.width = this.stage.stageWidth;
			_rollBG._bg.height = this.stage.stageHeight;
			
			// 閉じるボタンの実装
			_closeBtn = new CloseBtn();
			_closeBtn.addEventListener(MouseEvent.CLICK, closeBtnHandler, false, 0, true);
			this.addChild(_closeBtn);
			_closeBtn.x = this.stage.stageWidth - _closeBtn.width - 18;
			_closeBtn.y = 18;
			
			// ファイルのチェック
			checkFile(list);
			
			// 画面をフェードインする
			Tweener.addTween(this, { alpha:1, time:2, transition:Equations.easeInOutCubic } );
		}
		
		// 渡されたファイルのリストをチェックする
		private function checkFile(list:Array) {
			
			_imgNodeList = new Array();
			_imgLoadCnt = 0;
			_musicNodeList = new Array();
			_musicLoadCnt = 0;
			_otherList = new Array();
			
			for (var i:Number = 0; i < list.length; i++) {
				var name:String = list[i].name;
				
				// 拡張子よりファイルのタイプを分ける
				switch (checkSuffix(list[i])) {
					case 0:
						// 一般のファイル
						_otherList.push(list[i]);
						break;
					case 1:
						// 画像ファイルは背景に画像を読み込ませる
						var imgNode:ImgNode = new ImgNode();
						imgNode.init(File(list[i]));
						imgNode.addEventListener(Event.COMPLETE, loadFileCheck, false, 0, true);
						_imgNodeList.push(imgNode);
						_otherList.push(list[i]);
						break;
					case 2:
						// 音声ファイルは再生させ、音声ファイル用の表示を行う
						var musicNode:MusicNode = new MusicNode();
						musicNode.init(File(list[i]));
						musicNode.addEventListener(Event.COMPLETE, loadFileCheck, false, 0, true);
						_musicNodeList.push(musicNode);
						break;
				}
			}
			
			// 画像ファイル、音声ファイルがない場合はエンドロールを開始する
			if (_imgNodeList.length == 0 && _musicNodeList.length == 0) {
				startRoll();
			}
		}
		
		// ファイルの拡張子をチェックする
		private function checkSuffix(file:File):Number {
			var res:Number = 0;
			if (StringUtil.endsWith(file.name, ".jpg") ||
				StringUtil.endsWith(file.name, ".JPG") ||
				StringUtil.endsWith(file.name, ".jpeg") ||
				StringUtil.endsWith(file.name, ".JPEG") ||
				StringUtil.endsWith(file.name, ".png") ||
				StringUtil.endsWith(file.name, ".PNG") ||
				StringUtil.endsWith(file.name, ".gif") ||
				StringUtil.endsWith(file.name, ".GIF")) {
				res = 1;
			} else if (StringUtil.endsWith(file.name, ".mp3")) {
				res = 2;
			}
			return res;
		}
		
		// 読み込んだファイル数をチェックし、エンドロール開始まで待ち受ける。
		private function loadFileCheck(e:Event):void {
			switch (getQualifiedClassName(e.target)) {
				case "MusicNode":
					_musicLoadCnt++;
					break;
				case "ImgNode":
					_imgLoadCnt++;
					break;
			}
			
			if (_imgLoadCnt == _imgNodeList.length &&
				_musicLoadCnt == _musicNodeList.length) {
				startRoll();	
			}
		}
		
		// エンドロールの開始
		private function startRoll() {
			
			// 流れる文字を載せて移動するパネルの生成
			_panel = new Sprite();
			this.addChild(_panel);
			
			var posY:Number = 0;
			
			var subTitleTrash:TextField = generateMonoTxtField("---- trash ----");
			subTitleTrash.x = this.stage.width / 2 - subTitleTrash.textWidth / 2;
			_panel.addChild(subTitleTrash);
			subTitleTrash.y = posY;
			posY += 100;
			
			// ファイル名をパネルに表示する
			for (var i:Number = 0; i < _otherList.length; i++) {
				var file:File = _otherList[i] as File;
				var tmpInfo:Array = file.name.split(".");
				var suffix:String = tmpInfo[tmpInfo.length - 1] as String;
				var otherNode:OtherNode = new OtherNode();
				otherNode._title.text = suffix + " ファイル";
				otherNode._desc.text = file.name;
				_panel.addChild(otherNode);
				otherNode.x = this.stage.stageWidth / 2;
				otherNode.y = posY;
				posY += 100;
			}
			
			// 音声ファイルがある場合はパネルに音声ファイル用の表示をする。
			if (_musicNodeList.length > 0) {
				
				var subTitleMP3:TextField = generateMonoTxtField("---- mp3 ----");
				subTitleMP3.x = this.stage.stageWidth / 2 - subTitleMP3.textWidth / 2;
				_panel.addChild(subTitleMP3);
				subTitleMP3.y = posY;
				posY += 100;
				
				for (var j:Number = 0; j < _musicNodeList.length; j++) {
					var musicNode:MusicNode = _musicNodeList[j];
					_panel.addChild(musicNode);
					musicNode.y = posY;
					if (j % 2 == 0) {
						musicNode.x = this.stage.stageWidth / 2 - 260;
					} else {
						musicNode.x = this.stage.stageWidth / 2 + 20;
						posY += 150;
					}
				}
			}
			
			// 最後に END と表示する
			var subTitleEnd:TextField = generateMonoTxtField("END.");
			subTitleEnd.x = this.stage.width / 2 - subTitleEnd.textWidth / 2;
			subTitleEnd.y = posY + this.stage.stageHeight;
			_panel.addChild(subTitleEnd);
			
			// パネルの移動開始
			_panel.y = this.stage.stageHeight;
			_panel.addEventListener(Event.ENTER_FRAME, motion, false, 0 ,true);
			
			// 画像と音声の表示と再生
			showNextImg();
			playNextMP3();
		}
		
		// 見出し用の一行だけの文字列を記述したTextFieldを返す
		private function generateMonoTxtField(desc:String ):TextField {
			var txtField:TextField = new TextField();
			txtField.text = desc;
			var txtFmt:TextFormat = txtField.getTextFormat(0, txtField.length);
			txtFmt.color = 0xffffff;
			txtFmt.size = 18;
			txtField.setTextFormat(txtFmt);
			return txtField;
		}
		
		// MP3 を順番に再生
		private function playNextMP3(e:Event = null) {
			if (_musicNodeList.length == 0) return;
			if (_currentMusicNo >= _musicNodeList.length) _currentMusicNo = 0; 
			if (_currentMusic) _currentMusic.stop();
			var node:MusicNode = _musicNodeList[_currentMusicNo];
			_currentMusic = node.sound.play();
			_currentMusic.addEventListener(Event.SOUND_COMPLETE, playNextMP3);
			if (node.sound.length / 1000 > 60) {
				_currentMusic.soundTransform.volume = 0;
				Tweener.addTween(_currentMusic, { _sound_volume:1, time:10 } );	
			}
		}
		
		// 画像を順番に表示
		private function showNextImg() {
			if (_imgNodeList.length == 0) return;
			if (_currentImgNo >= _imgNodeList.length) _currentImgNo = 0; 
			if (_currentImg) _rollBG.removeChild(_currentImg);
			
			_currentImg = _imgNodeList[_currentImgNo] as ImgNode;
			_currentImgNo++;
			_rollBG.addChild(_currentImg);
			_currentImg.alpha = 0;
			
			ColorShortcuts._brightness_set(_currentImg, -3, []);
			Tweener.addTween(_currentImg, { _brightness:0, time:1, delay:1, transition:Equations.easeInOutCubic, onComplete:closeImg } );
			
			// 左上と右下に交互に表示
			if (_currentImgNo % 2 == 1) {
				_currentImg.x = 20;
				_currentImg.y = 20;
			} else {
				_currentImg.x = this.stage.stageWidth - _currentImg.width - 20;
				_currentImg.y = this.stage.stageHeight - _currentImg.height - 20;
			}
		}
		
		// 表示した画像をフェードアウトさせる
		private function closeImg() {
			Tweener.addTween(_currentImg, { _brightness:-3, time:1, delay:2, transition:Equations.easeInOutCubic, onComplete:showNextImg} );
		}
		
		// パネルの移動処理
		private function motion(e:Event):void {
			_panel.y -= 2;
			// 最後まで表示したら終了する
			if (_panel.y <= - _panel.height + this.stage.stageHeight / 2) {
				_panel.removeEventListener(Event.ENTER_FRAME, motion);
				endRoll();
			}
		}
		
		// エンドロールの終了を行う
		private function endRoll() {
			if (_currentMusic != null) {
				// 音声がある場合はフェードアウトさせる
				Tweener.addTween(_currentMusic, { _sound_volume:0, time:3 } );
			}
			
			// 全体をフェードアウトさせて閉じる
			Tweener.addTween(this, { alpha:0, time:2, delay:2, transition:Equations.easeInOutCubic,
										onComplete:closePanel } );
			
			// 画像と音声が再生され続けないようにリストを空にする
			_imgNodeList.splice(0, _imgNodeList.length);
			_musicNodeList.splice(0, _musicNodeList.length);
		}
		
		private function closePanel(e:MouseEvent = null):void {
			trace("closePanel");
			if (_currentMusic) _currentMusic.stop(); 
			_window.close();
		}
	}
}
