﻿package {
	
	import caurina.transitions.properties.*;
	import flash.data.*;
	import flash.desktop.*;
	import flash.display.*;
	import flash.events.*;
	import flash.filesystem.*;
	import flash.net.*;
	import flash.ui.*;
	import flash.utils.*;
	
	/**
	 * ...
	 * @author theodoorjp
	 */
	public class TrashRoll extends Sprite {
		
		public var _deleteBtn:MovieClip;
		public var _trashCan:MovieClip;
		private var _rollPanel:RollPanel;
		private var _trashNo:int = 0;
		
		/**
		 * ゴミ箱としての基本機能を実装
		 */
		public function TrashRoll() {
			
			// Tweener のショートカットを初期化
			SoundShortcuts.init();
			ColorShortcuts.init();
			
			// ごみの総数をカウント
			var info:ByteArray = EncryptedLocalStore.getItem("trashNo");
			if (info != null) _trashNo = info.readInt();
			setImg(_trashNo);
			
			//AIRアプリ内にドラッグインしたときに
			_trashCan.addEventListener(NativeDragEvent.NATIVE_DRAG_ENTER, dragEnterHandler);
			//AIRアプリ内にドロップしたときに
			_trashCan.addEventListener(NativeDragEvent.NATIVE_DRAG_DROP, dragDropHandler);
			
			//AIRアプリのドラッグ操作
			_trashCan.addEventListener(MouseEvent.MOUSE_DOWN, function(e:Event) {
					stage.nativeWindow.startMove();
				});
			
			// ボタンの初期化
			_deleteBtn.buttonMode = true;
			_deleteBtn.addEventListener(MouseEvent.CLICK, deleteAllItem);
		}
		
		// ゴミの数に合わせてゴミ箱の見た目とボタンの表示を切り替える
		private function setImg(no:Number):void {
			if (no == 0) {
				_trashCan.gotoAndStop(1);
			} else {
				_trashCan.gotoAndStop(2);
			}
			
			// 10 個以上ゴミが溜まったら捨てられるようになる。
			if (no >= 10 && !this.contains(_deleteBtn)) {
				this.addChild(_deleteBtn);
			} else if(no < 10 && this.contains(_deleteBtn)) {
				this.removeChild(_deleteBtn);
			}
		}
		
		//ドラッグイン
		private function dragEnterHandler(evt:NativeDragEvent):void {
			if (evt.clipboard.hasFormat(ClipboardFormats.FILE_LIST_FORMAT)) {
				//キャンバスへのドラッグインを許可
				NativeDragManager.acceptDragDrop(_trashCan);
			}
		}
		
		// AIRアプリ内にドロップ、ファイルを一時フォルダに移動
		private function dragDropHandler(evt:NativeDragEvent):void {
			
			var obj:Object = evt.clipboard.getData(ClipboardFormats.FILE_LIST_FORMAT);
			
			for (var i in obj) {
				var trash:File = File(obj[i]);
				_trashNo += countFileNo(trash);
				saveTrashInfo(_trashNo);
				var destination:File = File.applicationStorageDirectory;
				var removeName:String = isSameFile(destination, trash.name);
				destination = destination.resolvePath(removeName);
				trash.moveTo(destination);
			}
		}
		
		// ファイル数をカウント
		private function countFileNo(trash:File):Number {
			
			var cnt:Number = 0;
			
			// ファイルがディレクトリの場合再起処理を行う
			if (trash.isDirectory) {
				var list:Array = trash.getDirectoryListing();
				for (var i:Number = 0; i < list.length; i++ ) {
					cnt += countFileNo(File(list[i]));
				}
			} else {
				cnt++;
			}
			
			return cnt;
		}
		
		// ゴミの数を保存する。
		private function saveTrashInfo(no:Number):void {			
			_trashNo = no;
			var saveInfo:ByteArray = new ByteArray();
			saveInfo.writeInt(_trashNo);
			EncryptedLocalStore.setItem("trashNo", saveInfo);
			setImg(_trashNo);
		}
		
		// 同じ名前のファイルがある場合、名前を変更して返す
		private function isSameFile(file:File, name:String):String {
			
			var list:Array = file.getDirectoryListing();
			
			for (var i:Number = 0; i < list.length; i++) {
				var chkFile:File = File(list[i]);
				if (chkFile.name == name) {
					
					var nameList:Array = name.split(".");
					
					if (nameList.length > 1) {
						// 拡張子がある場合はその前に _copy と入れる
						name = name.substr(0, name.length - nameList[nameList.length - 1].length - 1)
								+ "_copy." + nameList[nameList.length - 1];
					} else {
						name = name.concat("_copy");
					}
					
					// 変更後同じ名前がないか再起処理
					name = isSameFile(file, name);
					break;
				}
			}
			
			return name;
		}
		
		// ファイルをゴミ箱へ移動し、エンドロールを作成する
		private function deleteAllItem(e:Event):void {
			_rollPanel = new RollPanel();
			_rollPanel.addEventListener(Event.CLOSE, moveToTrash);
			// エンドロールが開くのを待ってから初期化
			setTimeout(initRollPanel, 50);
		}
		
		//エンドロールの初期化を行う
		private function initRollPanel() {
			var trashList:Array = getTrashFileList(File.applicationStorageDirectory, new Array());
			_rollPanel.init(trashList);
		}
		
		// 全ファイルをリストにして返す
		private function getTrashFileList(trashDir:File, resList:Array):Array {
			
			var list:Array = trashDir.getDirectoryListing();
			
			for (var i:Number = 0; i < list.length; i++ ) {
				var trash:File = File(list[i]);
				if (trash.isDirectory) {
					// フォルダの場合、中のファイルを再起処理で返す 
					getTrashFileList(trash, resList);
				} else {
					resList.push(trash);
				}
			}
			
			return resList;
		}
		
		// 全てのファイルをゴミ箱に移動
		private function moveToTrash(e:Event):void {
			
			trace("move to trash");
			
			var list:Array = File.applicationStorageDirectory.getDirectoryListing();
			
			for (var i:Number = 0; i < list.length; i++) {
				File(list[i]).moveToTrashAsync();
			}
			saveTrashInfo(0);
		}
	}
}
